using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Imaging;  
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using System.Reflection;
using System.IO;
using System.Collections.ObjectModel;  

namespace VEarth
{
    public class VEarthMouseClickEventArgs : EventArgs
    {
        public double Latitude { get; set; }
        public double Longitude { get; set; }
        public object Id { get; set; }
        public bool AltKey { get; set; }
        public bool CtrlKey { get; set; }
        public bool ShiftKey { get; set; }

        public VEarthMouseClickEventArgs(double latitude, double longitude, object id, bool altKey, bool ctrlKey, bool shiftKey)
        {
            Latitude = latitude;
            Longitude = Longitude;
            Id = id;
            AltKey = altKey;
            CtrlKey = ctrlKey;
            ShiftKey = shiftKey;
        }
    }

    public class VEarthMouseClickWithButtonEventArgs : VEarthMouseClickEventArgs
    {
        public bool LeftButton { get; set; }
        public bool RightButton { get; set; }

        public VEarthMouseClickWithButtonEventArgs(double latitude, double longitude, object id, bool altKey, bool ctrlKey, bool shiftKey, bool leftButton, bool rightButton)
            : base(latitude, longitude, id, altKey, ctrlKey, shiftKey)
        {
            LeftButton = leftButton;
            RightButton = rightButton;
        }
    }

    public delegate void FinishedFindingLocations(object sender, EventArgs e);
    public delegate void MouseClick(object sender, VEarthMouseClickWithButtonEventArgs e);
    public delegate void MouseDoubleClick(object sender, VEarthMouseClickEventArgs e);
    public delegate void MouseMove(object sender, VEarthMouseClickWithButtonEventArgs e);
    public delegate void MouseWheel(object sender, VEarthMouseClickEventArgs e);
    public delegate void MouseOver(object sender, VEarthMouseClickEventArgs e);
    public delegate void MouseOut(object sender, VEarthMouseClickEventArgs e);
    
    [ComVisible(true)]
    public partial class ucVEarth : UserControl
    {
        private int _searchLocationIndex = 0;
        private bool _centerAtLastResult = false;
        private bool _displayResults = false;
        private bool _finishedFindingLocations = false;
        private Collection<SearchLocation> _locations = new Collection<SearchLocation>();
        private RouteDirections _routeDirections = null;
        
        public event FinishedFindingLocations VE_OnFinishedFindingLocations;
        public event MouseClick VE_OnMouseClick;
        public event MouseDoubleClick VE_OnMouseDoubleClick;
        public event MouseMove VE_OnMouseMove;
        public event MouseWheel VE_OnMouseWheel;
        public event MouseOver VE_OnMouseOver;
        public event MouseOut VE_OnMouseOut;


        public void TriggerClick(double latitude, double longitude, object id, bool altKey, bool ctrlKey, bool shiftKey, bool leftButton, bool rightButton)
        {
            if (VE_OnMouseClick != null)
            {
                VE_OnMouseClick(this, new VEarthMouseClickWithButtonEventArgs( latitude, longitude, id, altKey, ctrlKey, shiftKey, leftButton, rightButton));
            }
        }

        public void TriggerDoubleClick(double latitude, double longitude, object id, bool altKey, bool ctrlKey, bool shiftKey)
        {
            if (VE_OnMouseDoubleClick != null)
            {
                VE_OnMouseDoubleClick(this, new VEarthMouseClickEventArgs(latitude, longitude, id, altKey, ctrlKey, shiftKey));
            }
        }

        public void TriggerMouseMove(double latitude, double longitude, object id, bool altKey, bool ctrlKey, bool shiftKey, bool leftButton, bool rightButton)
        {
            if (VE_OnMouseMove != null)
            {
                VE_OnMouseMove(this, new VEarthMouseClickWithButtonEventArgs( latitude, longitude, id, altKey, ctrlKey, shiftKey, leftButton, rightButton));
            }
        }

        public void TriggerMouseWheel(double latitude, double longitude, object id, bool altKey, bool ctrlKey, bool shiftKey)
        {
            if (VE_OnMouseWheel != null)
            {
                VE_OnMouseWheel(this, new VEarthMouseClickEventArgs(latitude, longitude, id, altKey, ctrlKey, shiftKey));
            }
        }

        public void TriggerMouseOut(double latitude, double longitude, object id, bool altKey, bool ctrlKey, bool shiftKey)
        {
            if (VE_OnMouseOut != null)
            {
                VE_OnMouseOut(this, new VEarthMouseClickEventArgs( latitude, longitude, id, altKey, ctrlKey, shiftKey));
            }
        }

        public void TriggerMouseOver(double latitude, double longitude, object id, bool altKey, bool ctrlKey, bool shiftKey)
        {
            if (VE_OnMouseOver != null)
            {
                VE_OnMouseOver(this, new VEarthMouseClickEventArgs(latitude, longitude, id, altKey, ctrlKey, shiftKey));
            }
        }
        
        /// <summary>
        /// Constructor
        /// </summary>
        public ucVEarth()
        {
            InitializeComponent();

            if (!this.DesignMode)
            {
                using (StreamReader sr = new StreamReader(Assembly.GetExecutingAssembly().GetManifestResourceStream("VEarth.VirtualEarth.html")))
                {
                    string html = sr.ReadToEnd();
                    vEarthBrowser.DocumentText = html;
                }
                vEarthBrowser.ObjectForScripting = this;
                vEarthBrowser.Document.InvokeScript("startVE");
            }
        }

        /// <summary>
        /// Initializes Virtual Earth
        /// </summary>
        private void vEarthBrowser_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
        {
            if (!this.DesignMode)
            {
                vEarthBrowser.Document.InvokeScript("startVE");
            }
        }

        /// <summary>
        /// Resizes the map
        /// </summary>
        private void vEarthBrowser_Resize(object sender, EventArgs e)
        {
            if (!this.DesignMode && vEarthBrowser.Document != null && vEarthBrowser.Document.GetElementById("myMap") != null)
            {
                vEarthBrowser.Document.GetElementById("myMap").Style = "overflow: hidden; width: 100%; height: 100%;";
                vEarthBrowser.Document.InvokeScript("maxSize");
            }
        }

        /// <summary>
        /// Flag indicating that a call to VE_FindLocations is finished
        /// </summary>
        public bool FinishedFindingLocations
        {
            get
            {
                return _finishedFindingLocations;
            }
        }
        
        private DistanceUnit distanceUnit = DistanceUnit.Kilometers;

        [Browsable(true), 
        Description("The distance unit used."),
        DefaultValue(DistanceUnit.Kilometers),
        Category("Virtual Earth")]
        public DistanceUnit DistanceUnit
        {
            get { return distanceUnit; }
            set 
            { 
                distanceUnit = value;

                if (!this.DesignMode && vEarthBrowser.Document != null)
                {
                    vEarthBrowser.Document.InvokeScript("setDistanceUnit", new object[] { (int)distanceUnit });
                }
            }
        }

        private Disambiguation disambiguationMode = Disambiguation.Default;

        [Browsable(true),
        Description("The disambiguation mode."),
        DefaultValue(Disambiguation.Default),
        Category("Virtual Earth")]
        public Disambiguation DisambiguationMode
        {
            get { return disambiguationMode; }
            set
            {
                disambiguationMode = value;

                if (!this.DesignMode && vEarthBrowser.Document != null)
                {
                    vEarthBrowser.Document.InvokeScript("setDisambiguationMode", new object[] { (int)disambiguationMode });
                }
            }
        }

        private DashboardStyle dashboardStyle = DashboardStyle.Normal;

        [Browsable(true),
        Description("The size of the dashboard."),
        DefaultValue(DashboardStyle.Normal),
        Category("Virtual Earth")]
        public DashboardStyle DashboardStyle
        {
            get { return dashboardStyle; }
            set
            {
                dashboardStyle = value;

                if (!this.DesignMode && vEarthBrowser.Document != null)
                {
                    vEarthBrowser.Document.InvokeScript("setDashboardStyle", new object[] { (int)dashboardStyle });
                }
            }
        }

        private MapStyle mapStyle = MapStyle.Road;

        [Browsable(true),
        Description("Defines the way the map is shown."),
        DefaultValue(MapStyle.Road),
        Category("Virtual Earth")]
        public MapStyle MapStyle
        {
            get { return mapStyle; }
            set
            {
                mapStyle = value;

                if (!this.DesignMode && vEarthBrowser.Document != null)
                {
                    vEarthBrowser.Document.InvokeScript("setMapStyle", new object[] { (int)mapStyle });
                }
            }
        }

        private bool showDashboard = true;

        [Browsable(true),
        Description("Show or hide the dashboard."),
        DefaultValue(true),
        Category("Virtual Earth")]
        public bool ShowDashboard
        {
            get { return showDashboard; }
            set
            {
                showDashboard = value;

                if (!this.DesignMode && vEarthBrowser.Document != null)
                {
                    vEarthBrowser.Document.InvokeScript("setDashboardVisibility", new object[] { showDashboard });
                }
            }
        }

        private string mapLocation = ""; 

        [Browsable(true),
        Description("Set the location displayed on the map."),
        DefaultValue(""), 
        Category("Virtual Earth")]
        public string MapLocation
        {
            get { return mapLocation; }
            set
            {
                mapLocation = value;

                if (!this.DesignMode && vEarthBrowser.Document != null)
                {
                    vEarthBrowser.Document.InvokeScript("findLocation", new object[] { mapLocation });
                }
            }
        }

        private int zoomLevel = 10;

        [Browsable(true),
        Description("Set the zoomlevel. (1 to 19)"),
        DefaultValue(10),
        Category("Virtual Earth")]
        public int ZoomLevel
        {
            get 
            {
                if (!this.DesignMode && vEarthBrowser.Document != null)
                {
                    return (int)vEarthBrowser.Document.InvokeScript("getZoomLevel");
                }
                else
                {
                    return zoomLevel;
                }
            }
            set
            {
                zoomLevel = value;

                if (!this.DesignMode && vEarthBrowser.Document != null)
                {
                    vEarthBrowser.Document.InvokeScript("find", new object[] { zoomLevel });
                }
            }
        }

        public Collection<SearchLocation> SearchLocations
        {
            get { return _locations; }
        }
	
        /// <summary>
        /// Set zoom level of the map
        /// </summary>
        /// <param name="level">The level to zoom to (1 to 19)</param>
        public void SetZoomLevel(int level)
        {
            if (level < 1 || level > 19)
            {
                throw new Exception("Level must be a number between 1 and 19."); 
            }
            
            if (!this.DesignMode && vEarthBrowser.Document != null)
            {
                vEarthBrowser.Document.InvokeScript("setZoomLevel", new object[] { level });
            }
        }

        /// <summary>
        /// Zoom in 1 level on the map
        /// </summary>
        public void ZoomIn()
        {
            if (!this.DesignMode && vEarthBrowser.Document != null)
            {
                vEarthBrowser.Document.InvokeScript("zoomIn");
            }
        }

        /// <summary>
        /// Zoom out 1 level on the map
        /// </summary>
        public void ZoomOut()
        {
            if (!this.DesignMode && vEarthBrowser.Document != null)
            {
                vEarthBrowser.Document.InvokeScript("zoomOut");
            }
        }

        /// <summary>
        /// Add a shape layer to the map
        /// </summary>
        /// <param name="title">Title of the layers</param>
        /// <param name="description">Description of the layer</param>
        /// <remarks>Shapelayers are identified by their title</remarks> 
        public void AddShapeLayer(string title, string description)
        {
            if (!this.DesignMode && vEarthBrowser.Document != null)
            {
                vEarthBrowser.Document.InvokeScript("addShapeLayer", new object[] { title, description });
            }
        }

        /// <summary>
        /// Remove as shapelayer and its contents from the map
        /// </summary>
        /// <param name="title">The title of the layer to remove</param>
        /// <returns>false if the layer cannot be found</returns>
        /// <remarks>Shapelayers are identified by their title</remarks> 
        public bool DeleteShapeLayer(string title)
        {
            if (!this.DesignMode && vEarthBrowser.Document != null)
            {
                return (bool)vEarthBrowser.Document.InvokeScript("deleteShapeLayer", new object[] { title });
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Delete all shapelayers and their content from the map
        /// </summary>
        public void DeleteAllShapeLayers()
        {
            vEarthBrowser.Document.InvokeScript("deleteAllShapeLayers");
        }

        /// <summary>
        /// Set the visibility of a shapelayer
        /// </summary>
        /// <param name="title">The title of the shapelayer to modify</param>
        /// <param name="visible">true to show the shapelayer, false to hide</param>
        /// <returns>false if the layer cannot be found</returns>
        /// <remarks>Shapelayers are identified by their title</remarks> 
        public bool SetShapeLayerVisible(string title, bool visible)
        {
            return (bool)vEarthBrowser.Document.InvokeScript("setShapeLayerVisible", new object[] { title, visible });
        }

        /// <summary>
        /// Search for multiple locations and display their location on the map using a pushpin.
        /// </summary>
        /// <param name="locations">List of locations to search for</param>
        /// <param name="centerAtLastResult">if true, the map will be centered at the last found location</param> 
        /// <param name="displayResults">if true, results will be shown on the map as pushpins</param> 
        /// <remarks>
        /// Use this function to find locations by where or what. If the latitude and Longitude is known, use
        /// the AddPushPin function instead.
        /// After a succesful call the latitude and Longitude of the locations in the list is set
        /// If a location is not found, the latitude and Longitude will be null
        /// </remarks> 
        public void FindLocations(Collection<SearchLocation> locations, bool centerAtLastResult, bool displayResults, FinishedFindingLocations callbackFunction)
        {
            _locations = locations;
            _centerAtLastResult = centerAtLastResult;
            _displayResults = displayResults;

            if(locations.Count > 0)
            {
                _finishedFindingLocations = false;

                _searchLocationIndex = 0;
                SearchLocation searchLocation = locations[_searchLocationIndex];
                vEarthBrowser.Document.InvokeScript("find", new object[] { searchLocation.What, searchLocation.Where, centerAtLastResult });
            }

            if (callbackFunction != null)
            {
                VE_OnFinishedFindingLocations += new FinishedFindingLocations(callbackFunction);
            }
        }

        /// <summary>
        /// Callback function for the find call to the Virtual Earth control, adds the latitude and Longitude to the locations
        /// </summary>
        /// <param name="latitude">Latitude of the found location, or null when the location is not found</param>
        /// <param name="longitude">Longitude of the found location, or null when the location is not found</param>
        /// <param name="errorMessage">Error message if an error has occured</param>
        /// <remarks>This method should not be called directly, instead use the VE_DisplayLocations(..) method</remarks> 
        public void OnJavascriptLocationFound(object latitude, object longitude, object errorMessage)
        {
            _locations[_searchLocationIndex].Latitude = (double?)latitude;
            _locations[_searchLocationIndex].Longitude = (double?)longitude;

            if (_locations != null && _locations.Count > _searchLocationIndex + 1)
            {
                SearchLocation searchLocation = _locations[++_searchLocationIndex];
                vEarthBrowser.Document.InvokeScript("find", new object[] { searchLocation.What, searchLocation.Where });
            }
            else if(_locations != null && _displayResults)
            {   // all locations are search for, now display the results if specified
                DisplayLocations();
                _finishedFindingLocations = true;
            }
            else if (_centerAtLastResult && _locations.Count > 0 && !_displayResults)
            {   // if the results will not be displayed but the map should be centered at the last location, then do that now
                SetCenter(_locations[_locations.Count - 1]);
                _finishedFindingLocations = true;
            }
            else
            {
                _finishedFindingLocations = true;
            }

            // Trigger public event
            if (_finishedFindingLocations)
            {
                if(VE_OnFinishedFindingLocations!=null)
                    VE_OnFinishedFindingLocations(this, new EventArgs());
            }
        }

        /// <summary>
        /// Add pushpins for all locations in the collection
        /// </summary>
        private void DisplayLocations()
        {
            foreach (SearchLocation location in _locations)
            {
                if (location.Latitude != null && location.Longitude != null)
                {
                    object id = vEarthBrowser.Document.InvokeScript("addPushPin", 
                        new object[] { 
                            location.Latitude, 
                            location.Longitude, 
                            location.PushPinTitle, 
                            location.PushPinLayer, 
                            location.PushPinImage, 
                            location.PushPinDescription });

                    location.ID = id == null ? "" : id.ToString();
                }
            }

           // if specified, the last result will be centered on the map
            if (_centerAtLastResult && _locations.Count > 0)
            {
                SetCenter(_locations[_locations.Count - 1]);
            }
        }

        /// <summary>
        /// Centers the map at the specified location (based on latitude/Longitude or where/what)
        /// </summary>
        /// <param name="location">The location the map will center at</param>
        public void SetCenter(SearchLocation location)
        {
            // set center by coordinates...
            if (location.Latitude != null && location.Longitude != null)
            {
                vEarthBrowser.Document.InvokeScript("setCenter",
                        new object[] 
                        { 
                            location.Longitude, 
                            location.Latitude 
                        });
            } // ...or by what / where
            else if (location.Where != String.Empty || location.What != String.Empty)
            {
                _locations = new Collection<SearchLocation>();
                SearchLocations.Add(location);
                FindLocations(_locations, true, false, null);    
            }
        }

        /// <summary>
        /// Initiate route calculations
        /// </summary>
        /// <param name="locations">The waypoints of the route (max 25)</param>
        /// <param name="displayRoute">Display the route on the map yes or no</param>
        /// <param name="setBestMapView">Center the map to have best view of the route yes or no</param>
        /// <param name="routeMode">Walking or driving directions</param>
        /// <param name="distanceUnit">Miles or kilometers</param>
        /// <param name="routeOptimizeFactor">Optimize for minimal time or minimal distance</param> 
        /// <returns>RouteDirection object </returns> 
        /// <remarks>If the routeOptimizeFactor is set to Time and the RouteMode is set to walking, then an empty object is returned.</remarks> 
        public RouteDirections GetDirections(List<SearchLocation> locations, bool displayRoute, bool setBestMapView, RouteModeEnum routeMode, DistanceUnit distanceUnit, RouteOptimize routeOptimizeFactor)
        {
            Collection<SearchLocation> locationsWithLatLongCoordinates = new Collection<SearchLocation>();
            Collection<SearchLocation> locationsWithoutLatLongCoordinates = new Collection<SearchLocation>();
  
            if (locations.Count > 25)
            {
                throw new Exception("Maximum number of waypoints in a route is 25."); 
            }

            // split locations in those with and without latitude and longtidude information
            foreach (SearchLocation location in locations)
            {
                if (location.Latitude != null && location.Longitude != null)
                {
                    locationsWithLatLongCoordinates.Add(location);
                }
                else
                {
                    locationsWithoutLatLongCoordinates.Add(location);  
                }
            }

            // if there are locations without latitude and Longitude information, find the lat/long coordinates using where or what
            if (locationsWithoutLatLongCoordinates.Count > 0)
            {
                FindLocations(locationsWithoutLatLongCoordinates, false, false, null);

                while (!_finishedFindingLocations)
                {
                    Application.DoEvents();
                }

                foreach(SearchLocation sl in _locations)
                    locationsWithLatLongCoordinates.Add(sl);   
            }

            vEarthBrowser.Document.InvokeScript("clearRouteWaypoints");

            // add the waypoints by latitude and Longitude
            foreach (SearchLocation searchLocation in locationsWithLatLongCoordinates)
            {
                if (searchLocation.Latitude != null && searchLocation.Longitude != null)
                {
                    vEarthBrowser.Document.InvokeScript("addRouteWaypoint",
                        new object[] 
                        { 
                            searchLocation.Longitude, 
                            searchLocation.Latitude 
                        });
                }
            }

            _routeDirections = null;

            object waypoints = vEarthBrowser.Document.InvokeScript("getDirections", new object[] { displayRoute, setBestMapView, (int)routeMode, distanceUnit, (int)routeOptimizeFactor });

            // wait for asynchronous call to getDirections callback function to finish
            while (_routeDirections == null)
            {
                Application.DoEvents();  
            }

            return _routeDirections;
        }

        /// <summary>
        /// Callback function for VE_GetDirections
        /// </summary>
        /// <param name="time">Total time in seconds</param>
        /// <param name="distance">Total distance in miles or kilometers</param>
        /// <param name="description">Route description</param>
        public void OnDirectionsFinished(object duration, object distance, object description)
        {
            if (distance == null) throw new ArgumentNullException("distance");
            if (duration == null) throw new ArgumentNullException("duration");
            if (description == null) throw new ArgumentNullException("description");

            System.Globalization.NumberFormatInfo nfi = new System.Globalization.NumberFormatInfo();
            nfi.NumberDecimalSeparator = ".";
                  
            _routeDirections = new RouteDirections(double.Parse(duration.ToString()), decimal.Parse(distance.ToString(), nfi));

            foreach (string routeLeg in ((string)description).Trim('|').Split('|'))
            {
                string[] routeLegDetails = routeLeg.Split('~');  
                _routeDirections.AddRouteLeg(new RouteLeg(routeLegDetails[0], double.Parse(routeLegDetails[2]), decimal.Parse(routeLegDetails[1], nfi)));   
            }
        }

        /// <summary>
        /// Add a pushpin to the map based on Latitude and Longitude coordinates
        /// </summary>
        /// <param name="location">The location and appearance of the pushpin</param>
        /// <returns>The unique identifier of the added puspin (empty when to location could not be found)</returns>
        public string AddPushPin(SearchLocation location)
        {
            if (location != null)
            {

                if (location.Latitude == null && location.Longitude == null)
                {
                    throw new Exception("Specify the Latitude and Longitude of the pushpin.");
                }

                object id = vEarthBrowser.Document.InvokeScript("addPushPin",
                            new object[] { 
                            location.Latitude, 
                            location.Longitude, 
                            location.PushPinTitle, 
                            location.PushPinLayer, 
                            location.PushPinImage, 
                            location.PushPinDescription });

                location.ID = id == null ? "" : id.ToString();

                return location.ID;
            }
            throw new ArgumentNullException("location");
        }

        /// <summary>
        /// Add a line to the map between the given points
        /// </summary>
        /// <param name="locations">A list of points that make up the line</param>
        /// <returns>The unique identifier of the added line</returns>
        /// <remarks>Location that cannot be resolved to Lat/Long coordinates will be excluded</remarks> 
        public string AddShape(ShapeType shape, List<SearchLocation> pointsOfShape, string title, string description, string layer, int linewidth)
        {
            Collection<SearchLocation> locationsWithLatLongCoordinates = new Collection<SearchLocation>();
            Collection<SearchLocation> locationsWithoutLatLongCoordinates = new Collection<SearchLocation>();

            // split locations in those with and without latitude and longtidude information
            foreach (SearchLocation location in pointsOfShape)
            {
                if (location.Latitude != null && location.Longitude != null)
                {
                    locationsWithLatLongCoordinates.Add(location);
                }
                else
                {
                    locationsWithoutLatLongCoordinates.Add(location);
                }
            }

            // if there are locations without latitude and Longitude information, find the lat/long coordinates using where or what
            if (locationsWithoutLatLongCoordinates.Count > 0)
            {
                FindLocations(locationsWithoutLatLongCoordinates, false, false, null);

                while (!_finishedFindingLocations)
                {
                    Application.DoEvents();
                }

                foreach(SearchLocation sl in _locations)
                    locationsWithLatLongCoordinates.Add(sl);
            }

            vEarthBrowser.Document.InvokeScript("clearRouteWaypoints");

            // add the points by latitude and Longitude
            foreach (SearchLocation searchLocation in locationsWithLatLongCoordinates)
            {
                if (searchLocation.Latitude != null && searchLocation.Longitude != null)
                {
                    vEarthBrowser.Document.InvokeScript("addRouteWaypoint",
                        new object[] 
                        { 
                            searchLocation.Longitude, 
                            searchLocation.Latitude 
                        });
                }
            }

            object id = vEarthBrowser.Document.InvokeScript(shape == ShapeType.Polyline ? "addPolyline" : "addPolygon", 
                new object[] 
                {  
                    title,
                    description,
                    layer,
                    linewidth
                });

            return id == null ? "" : id.ToString();
        }

        /// <summary>
        /// Removes a pushpin from the map
        /// </summary>
        /// <param name="id">The unique identifier of the pushpin to remove</param>
        public void DeletePushPin(string id)
        {
            vEarthBrowser.Document.InvokeScript("deletePushPin", new object[] { id });  
        }

        /// <summary>
        /// Delete all pushpins, lines and polygones from the map
        /// </summary>
        public void DeleteAllShapes()
        {
            vEarthBrowser.Document.InvokeScript("deleteAllShapes");  
        }

        /// <summary>
        /// Delete current route directions from the map
        /// </summary>
        public void DeleteDirections()
        {
            vEarthBrowser.Document.InvokeScript("deleteDirections");
        }

        ///// <summary>
        ///// Save the exact appearance of the map as an image
        ///// </summary>
        ///// <param name="format">Image format (jpg, gif, png etc)</param>
        ///// <param name="path">Filename of the image. Existing files will be overwritten.</param>
        //public void SaveMapImage(ImageFormat format, string path)
        //{
        //    FormPrint.Save(this, format, path);  
        //}

    }
}